/* Provide gettimeofday
   1. for systems that don't have it,
   2. for some systems where gettimeofday clobbers the static buffer that
      localtime uses for it's return value.  The gettimeofday function from
      Mac OS X 10.0.4, i.e. Darwin 1.3.7 has this problem.
      The tzset replacement is necessary for at least Solaris 2.5, 2.5.1, and
      2.6.

   Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

/* written by Jim Meyering */

#include <config.h>

/* Specification.  */
#include "gettimeofday.h"

#include <sys/types.h>
#include <stdlib.h>

#if HAVE_SYS_TIMEB_H
# include <sys/timeb.h>
#endif

#if GETTIMEOFDAY_CLOBBERS_LOCALTIME

static struct tm *localtime_buffer_addr;

/* This is a wrapper for localtime.  It is used only on systems for which
   gettimeofday clobbers the static buffer used for localtime's result.

   On the first call, record the address of the static buffer that
   localtime uses for its result.  */

struct tm *
localtime (const time_t *timep)
#undef localtime
{
  extern struct tm *localtime (const time_t *);
  struct tm *tm = localtime (timep);

  if (! localtime_buffer_addr)
    localtime_buffer_addr = tm;

  return tm;
}

/* Same as above, since gmtime and localtime use the same buffer.  */
struct tm *
gmtime (const time_t *timep)
#undef gmtime
{
  extern struct tm *gmtime (const time_t *);
  struct tm *tm = gmtime (timep);

  if (! localtime_buffer_addr)
    localtime_buffer_addr = tm;

  return tm;
}

/* This is a wrapper for tzset. It is used only on systems for which
   tzset may clobber the static buffer used for localtime's result.
   Save and restore the contents of the buffer used for localtime's
   result around the call to tzset.  */
void
tzset (void)
#undef tzset
{
  extern struct tm *localtime (const time_t *);
  extern void tzset (void);
  struct tm save;

  if (! localtime_buffer_addr)
    {
      time_t t = 0;
      localtime_buffer_addr = localtime (&t);
    }

  save = *localtime_buffer_addr;
  tzset ();
  *localtime_buffer_addr = save;
}

#endif

/* This is a wrapper for gettimeofday.
   It is used only on systems that lack this function, or for whose
   implementation of this function causes problems. */

int
gettimeofday (struct timeval *restrict tv, void *restrict tz)
#undef gettimeofday
{
#if HAVE_GETTIMEOFDAY
  extern int gettimeofday (/* unspecified arguments */);
# if GETTIMEOFDAY_CLOBBERS_LOCALTIME
  extern struct tm *localtime (const time_t *);

  /* Save and restore the contents of the buffer used for localtime's result
     around the call to gettimeofday. */
  struct tm save;
  int result;

  if (! localtime_buffer_addr)
    {
      time_t t = 0;
      localtime_buffer_addr = localtime (&t);
    }

  save = *localtime_buffer_addr;
  result = gettimeofday (tv, tz);
  *localtime_buffer_addr = save;

  return result;

# else

  return gettimeofday (tv, tz);

# endif
#else

  /* The clock does not have microsecond resolution, so get the maximum
     possible value for the current time that is consistent with the
     reported clock.  That way, files are not considered to be in the
     future merely because their time stamps have higher resolution
     than the clock resolution.  */

# if HAVE__FTIME

  struct _timeb timebuf;

  _ftime (&timebuf);
  tv->tv_sec = timebuf.time;
  tv->tv_usec = timebuf.millitm * 1000 + 999;

  return 0;

# else

  time_t t = time (NULL);

  if (t == (time_t) -1)
    return -1;
  tv->tv_sec = t;
  tv->tv_usec = 999999;

  return 0;

# endif
#endif
}
